package net.pingex.dcf.commands;

import sx.blah.discord.api.IDiscordClient;
import sx.blah.discord.handle.obj.IChannel;
import sx.blah.discord.handle.obj.IMessage;
import sx.blah.discord.handle.obj.IUser;
import java.util.List;

/**
 * A Context contains all informations related to a command invocation, like the Command being invoked, its provided arguments and the originating message.
 */
public class Context
{
	/**
	 * The command being executed
	 */
	private Command command;

	/**
	 * Arguments provided
	 */
	private List<String> arguments;

	/**
	 * User to audit against
	 */
	private IUser user;

	/**
	 * Channel to audit against
	 */
	private IChannel channel;

	/**
	 * The originating reference message
	 */
	private IMessage originatingMessage;

	/**
	 * Discord client for context
	 */
	private IDiscordClient client;

	/**
	 * Construct a context using the provided message. user and channel will be derived from the message.
	 * @param command Command being executed.
	 * @param arguments Its arguments.
	 * @param originatingMessage The messaged used to invoke the command.
	 * @param client Discord client.
	 */
	public Context(Command command, List<String> arguments, IMessage originatingMessage, IDiscordClient client)
	{
		this.command = command;
		this.arguments = arguments;
		this.user = originatingMessage.getAuthor();
		this.channel = originatingMessage.getChannel();
		this.originatingMessage = originatingMessage;
		this.client = client;
	}

	/**
	 * Construct a context using explicit user and channel. The provided message is here for reference.
	 * @param command Command being executed.
	 * @param arguments Its arguments.
	 * @param user User invoking the command.
	 * @param channel Channel used to invoke the command.
	 * @param originatingMessage The messaged used to invoke the command.
	 * @param client Discord client.
	 */
	public Context(Command command, List<String> arguments, IUser user, IChannel channel, IMessage originatingMessage, IDiscordClient client)
	{
		this.command = command;
		this.arguments = arguments;
		this.user = user;
		this.channel = channel;
		this.originatingMessage = originatingMessage;
		this.client = client;
	}

	public Command getCommand()
	{
		return command;
	}

	public List<String> getArguments()
	{
		return arguments;
	}

	public IUser getUser()
	{
		return user;
	}

	public IChannel getChannel()
	{
		return channel;
	}

	public IMessage getOriginatingMessage()
	{
		return originatingMessage;
	}

	public IDiscordClient getClient()
	{
		return client;
	}
}
