package net.pingex.dcf.commands.parser;

import java.util.List;

/**
 * General interface for a command parser.
 */
public interface ICommandParser
{
	/**
	 * Checks the syntax of a command.
	 * @param input Input String, ie. the message content.
	 * @return `true` if the string is a command, `false` otherwise.
	 */
	boolean checkSyntax(String input);

	/**
	 * Gives the command part of a command string.
	 * @param input Input String, ie. the message content.
	 * @return The command part of the inputted string.
	 * @throws ParserException whether the inputted string cannot be parsed for a command.
	 */
	String parseCommand(String input) throws ParserException;

	/**
	 * Gives the arguments list of a command string.
	 * @param input Input String, ie. the message content.
	 * @return The arguments list of the inputted string.
	 * @throws ParserException whether the inputted string cannot be parsed for a command.
	 */
	List<String> parseArguments(String input) throws ParserException;
}
