package net.pingex.dcf.core;

import net.pingex.dcf.commands.CommandHandler;
import net.pingex.dcf.modularity.PluginWrapper;
import net.pingex.dcf.modularity.events.EventManager;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import sx.blah.discord.api.events.EventSubscriber;
import sx.blah.discord.handle.impl.events.*;

/**
 * Very core events handler, should be running independently from the whole events manager thingy.
 * It handles things like auto reconnect after gateway disconnection, etc.
 */
public class CoreEventsHandler
{
	/**
	 * Logger
	 */
	private static Logger LOGGER = LogManager.getLogger(CoreEventsHandler.class);

	@EventSubscriber
	public static void onReady(ReadyEvent event)
	{
		LOGGER.info("Connection with user #{} ready.", event.getClient().getOurUser().getID());
		EventManager.getInstance().updateConnectionHandlers();
	}

	@EventSubscriber
	public static void onDiscordDisconnected(DiscordDisconnectedEvent event)
	{
		LOGGER.warn("Discord connection with user #{} lost ({}).", event.getClient().getOurUser().getID(), event.getReason());
	}

	@EventSubscriber
	public static void onDiscordReconnected(DiscordReconnectedEvent event)
	{
		LOGGER.info("Gateway connection reconnected (user #{}).", event.getClient().getOurUser().getID());
	}

	@EventSubscriber
	public static void onMessageSend(MessageSendEvent event)
	{
		LOGGER.trace("Sent message to channel #{}.", event.getMessage().getChannel().getID());
	}

	@EventSubscriber
	public static void onMention(MentionEvent event)
	{
		LOGGER.trace("Received mention from channel #{}.", event.getMessage().getChannel().getID());
	}

	@EventSubscriber
	public static void onMessageReceived(MessageReceivedEvent event)
	{
		LOGGER.trace("Received message from channel #{}.", event.getMessage().getChannel().getID());
		CommandHandler.handle(event);
	}

	@EventSubscriber
	public static void onGuildUnavailable(GuildUnavailableEvent event)
	{
		LOGGER.warn("Guild #{} is unavailable.", event.getGuildID());
	}

	@EventSubscriber
	public static void onGuildCreate(GuildCreateEvent event)
	{
		LOGGER.info("Joined guild #{}.", event.getGuild().getID());
	}

	/**
	 * Called whenever a new plugin has been preloaded.
	 * @param plugin Target plugin
	 */
	public static void pluginPreloaded(PluginWrapper plugin)
	{

	}

	/**
	 * Called whenever a new plugin has been laoded.
	 * @param plugin Target plugin
	 */
	public static void pluginLoaded(PluginWrapper plugin)
	{

	}

	/**
	 * Called whenever a plugin is running.
	 * @param plugin Target plugin
	 */
	public static void pluginRunning(PluginWrapper plugin)
	{
		EventManager.getInstance().checkAndRegister(plugin); // Hook: event handlers
		CommandHandler.registerPlugin(plugin); // Hook: commands handler
	}

	/**
	 * Called whenever a plugin has stopped.
	 * @param plugin Target plugin
	 */
	public static void pluginStopped(PluginWrapper plugin)
	{
		EventManager.getInstance().unregister(plugin); // Hook: event handlers
		CommandHandler.unregisterPlugin(plugin); // Hook: commands handler
	}

	/**
	 * Called whenever a plugin has unloaded.
	 * @param plugin Target plugin
	 */
	public static void pluginUnloaded(PluginWrapper plugin)
	{

	}
}
