package net.pingex.dcf.core;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import sx.blah.discord.api.ClientBuilder;
import sx.blah.discord.api.IDiscordClient;
import sx.blah.discord.util.DiscordException;
import java.util.HashSet;
import java.util.Set;

/**
 * Storage of all active connection to Discord gateway
 */
public class GatewayConnectionManager
{
	/**
	 * Main connection
	 */
	private static IDiscordClient connection;

	/**
	 * List of registered listeners as we cannot access registered listeners in IDC dispatcher.
	 */
	private static Set<Object> registeredListeners = new HashSet<>();

	/**
	 * Logger
	 */
	private static final Logger LOGGER = LogManager.getLogger(GatewayConnectionManager.class);

	/**
	 * Registers a connection and login automatically, called once in main
	 * @param builder Builder filled and ready to login
	 */
	public static void registerConnection(ClientBuilder builder)
	{
		LOGGER.info("Registering new connection");
		try
		{
			connection = builder.login();
			connection.getDispatcher().registerListener(CoreEventsHandler.class); // Register the DCF core events handler needed for core operations and logging.
		}
		catch (DiscordException e)
		{
			LOGGER.warn("Failed to login to Discord.", e);
		}
	}

	/**
	 * Update listeners for target connection
	 * @param refListeners Reference listeners list
	 */
	public static void updateListeners(Set<Class<?>> refListeners)
	{
		LOGGER.debug("Updating connection listeners.");

		// Case 1: item is not registered in IDC
		long toRegister = refListeners.stream().filter(item -> !registeredListeners.contains(item)).count();
		refListeners.stream().filter(item -> !registeredListeners.contains(item)).forEach(item ->
		{
			connection.getDispatcher().registerListener(item);
			registeredListeners.add(item);
		});

		// Case 2: item is registered, but shouldn't be
		long toUnregister = registeredListeners.stream().filter(item -> !refListeners.contains(item)).count();
		registeredListeners.stream().filter(item -> !refListeners.contains(item)).forEach(item ->
		{
			connection.getDispatcher().unregisterListener(item);
			registeredListeners.remove(item);
		});

		LOGGER.debug("Update done. diff: +{} -{}", toRegister, toUnregister);
	}

	/**
	 * Reconnect a disconnected gateway connection
	 */
	@Deprecated
	public static void reconnect()
	{
		try
		{
			connection.login();
		}
		catch (DiscordException e)
		{
			LOGGER.warn("User #" + connection.getOurUser().getID() + " failed to reconnect to the gateway.", e);
		}
	}
}
